package CalculusAndVectors;

import java.lang.Math;

/* "Unitary" basis = "orthonormal vectors".
*/

public class UBasis {

  public DVector e[] = new DVector[3];
  public UBasis ( DVector e0, DVector e1, DVector e2 ) {
       this.e[0] = e0;
       this.e[1] = e1;
       this.e[2] = e2;
  }

  //Copies existing UBasis:
  public UBasis ( UBasis u ) {
       this.e[0] = new DVector(u.e[0].x, u.e[0].y, u.e[0].z);
       this.e[1] = new DVector(u.e[1].x, u.e[1].y, u.e[1].z);
       this.e[2] = new DVector(u.e[2].x, u.e[2].y, u.e[2].z);
  }


  public static void Assign( UBasis from, UBasis to ) {
         DVector.Assign( from.e[0], to.e[0] );                 
         DVector.Assign( from.e[1], to.e[1] );
         DVector.Assign( from.e[2], to.e[2] );
  }


  //Rotate Basis: Counter clockwise direction is positive.
  public void RotateAroundZ( double CFI, double SFI ) {
        
       double e0X = e[0].x * CFI + e[1].x * SFI;
       double e0Y = e[0].y * CFI + e[1].y * SFI;
       double e0Z = e[0].z * CFI + e[1].z * SFI;

       double e1X = e[0].x * (-SFI) + e[1].x * CFI;
       double e1Y = e[0].y * (-SFI) + e[1].y * CFI;
       double e1Z = e[0].z * (-SFI) + e[1].z * CFI;

       e[0].x = e0X;
       e[0].y = e0Y;
       e[0].z = e0Z;

       e[1].x = e1X;
       e[1].y = e1Y;
       e[1].z = e1Z;

       //e[2] is not changed; 

  }
 
  //Rotate Basis: Counter clockwise direction is positive.
  public void RotateAroundX( double CTheta, double STheta ) {
        
       double e1X = e[1].x * CTheta + e[2].x * STheta;
       double e1Y = e[1].y * CTheta + e[2].y * STheta;
       double e1Z = e[1].z * CTheta + e[2].z * STheta;

       double e2X = e[1].x * (-STheta) + e[2].x * CTheta;
       double e2Y = e[1].y * (-STheta) + e[2].y * CTheta;
       double e2Z = e[1].z * (-STheta) + e[2].z * CTheta;

       e[1].x = e1X;
       e[1].y = e1Y;
       e[1].z = e1Z;

       e[2].x = e2X;
       e[2].y = e2Y;
       e[2].z = e2Z;

       //e[0] is not changed

 }

 //Find angle psy to u from this basis.
 //Drawback. Creates an extra vector v which has t,p defined.
 public DVector PsyTauDefined( DVector u ) {

        double x = u.x;
        double y = u.y;
        double z = u.z;

        //Fist, rotate u by transformation opposite this basis T.
        //Columns e[0],e[1],e[2] comprise matrix T from global bais to e.
        //Because coordinates transform by inverse matrix T~,
        //which is transpose to T, we apply rows of T~ to 
        //coordinates of u:
        DVector v = new DVector( 
               x * e[0].x + y * e[0].y + z * e[0].z, //x
               x * e[1].x + y * e[1].y + z * e[1].z, //y
               x * e[2].x + y * e[2].y + z * e[2].z
            );

        v.FindSphericalCoord();
        return v;  

 }
           
}       